@@ -2,7 +2,8 @@ import Vue from 'vue';
 
 class AppState {
   constructor() {
-    this.visibleGroups = {}
+    this.visibleGroups = {};
+    this.timeFormat = "absolute";
   }
 
   isGroupVisible(group) {
@@ -12,6 +13,10 @@ class AppState {
   setGroupVisibility(group, visible) {
     Vue.set(this.visibleGroups, group.id, visible);
   }
+
+  setTimeFormat(timeFormat) {
+    this.timeFormat = timeFormat;
+  }
 }
 
 const appState = new AppState()
