@@ -10,43 +10,53 @@ called like::
 from __future__ import annotations
 
 import re
-from operator import methodcaller
 from typing import Any, Callable, Dict, Union
 
-from pyinstrument.frame import BaseFrame, Frame, FrameGroup, SelfTimeFrame
+from pyinstrument.frame import SELF_TIME_FRAME_IDENTIFIER, Frame, FrameGroup
+from pyinstrument.frame_ops import combine_frames, delete_frame_from_tree
 
 # pyright: strict
 
 
-ProcessorType = Callable[..., Union[BaseFrame, None]]
+ProcessorType = Callable[..., Union[Frame, None]]
 ProcessorOptions = Dict[str, Any]
 
 
-def remove_importlib(frame: BaseFrame | None, options: ProcessorOptions) -> BaseFrame | None:
+def remove_importlib(frame: Frame | None, options: ProcessorOptions) -> Frame | None:
     """
     Removes ``<frozen importlib._bootstrap`` frames that clutter the output.
     """
     if frame is None:
         return None
 
-    if not isinstance(frame, Frame):
-        return frame
-
     for child in frame.children:
         remove_importlib(child, options=options)
 
         if child.file_path and "<frozen importlib._bootstrap" in child.file_path:
-            # remove this node, moving the self_time and children up to the parent
-            frame.self_time += child.self_time
-            frame.add_children(child.children, after=child)
-            child.remove_from_parent()
+            delete_frame_from_tree(child, replace_with="children")
 
     return frame
 
 
-def aggregate_repeated_calls(
-    frame: BaseFrame | None, options: ProcessorOptions
-) -> BaseFrame | None:
+def remove_tracebackhide(frame: Frame | None, options: ProcessorOptions) -> Frame | None:
+    """
+    Removes frames that have set a local `__tracebackhide__` (e.g.
+    `__tracebackhide__ = True`), to hide them from the output.
+    """
+    if frame is None:
+        return None
+
+    for child in frame.children:
+        remove_tracebackhide(child, options=options)
+
+        if child.has_tracebackhide:
+            # remove this node, moving the self_time and children up to the parent
+            delete_frame_from_tree(child, replace_with="children")
+
+    return frame
+
+
+def aggregate_repeated_calls(frame: Frame | None, options: ProcessorOptions) -> Frame | None:
     """
     Converts a timeline into a time-aggregate summary.
 
@@ -58,22 +68,15 @@ def aggregate_repeated_calls(
     if frame is None:
         return None
 
-    children_by_identifier: dict[str, BaseFrame] = {}
+    children_by_identifier: dict[str, Frame] = {}
 
     # iterate over a copy of the children since it's going to mutate while we're iterating
     for child in frame.children:
         if child.identifier in children_by_identifier:
             aggregate_frame = children_by_identifier[child.identifier]
 
-            # combine the two frames, putting the children and self_time into the aggregate frame.
-            aggregate_frame.self_time += child.self_time
-            if child.children:
-                if not isinstance(aggregate_frame, Frame):
-                    raise Exception("cannot aggregate children into a DummyFrame")
-                aggregate_frame.add_children(child.children)
-
-            # remove this frame, it's been incorporated into aggregate_frame
-            child.remove_from_parent()
+            # combine child into aggregate frame, removing it from the tree
+            combine_frames(child, into=aggregate_frame)
         else:
             # never seen this identifier before. It becomes the aggregate frame.
             children_by_identifier[child.identifier] = child
@@ -83,16 +86,13 @@ def aggregate_repeated_calls(
         aggregate_repeated_calls(child, options=options)
 
     # sort the children by time
-    # it's okay to use the internal _children list, sinde we're not changing the tree
-    # structure.
-    frame._children.sort(key=methodcaller("time"), reverse=True)  # type: ignore # noqa
+    # we use the internal _children list, because we need to mutate it
+    frame._children.sort(key=lambda c: c.time, reverse=True)  # type: ignore # noqa
 
     return frame
 
 
-def group_library_frames_processor(
-    frame: BaseFrame | None, options: ProcessorOptions
-) -> BaseFrame | None:
+def group_library_frames_processor(frame: Frame | None, options: ProcessorOptions) -> Frame | None:
     """
     Groups frames that should be hidden into :class:`FrameGroup` objects,
     according to ``hide_regex`` and ``show_regex`` in the options dict, as
@@ -117,7 +117,7 @@ def group_library_frames_processor(
     hide_regex: str | None = options.get("hide_regex")
     show_regex: str | None = options.get("show_regex")
 
-    def should_be_hidden(frame: BaseFrame):
+    def should_be_hidden(frame: Frame):
         frame_file_path = frame.file_path or ""
 
         should_show = (show_regex is not None) and re.match(show_regex, frame_file_path)
@@ -131,7 +131,7 @@ def group_library_frames_processor(
 
         return not frame.is_application_code
 
-    def add_frames_to_group(frame: BaseFrame, group: FrameGroup):
+    def add_frames_to_group(frame: Frame, group: FrameGroup):
         group.add_frame(frame)
         for child in frame.children:
             if should_be_hidden(child):
@@ -150,8 +150,8 @@ def group_library_frames_processor(
 
 
 def merge_consecutive_self_time(
-    frame: BaseFrame | None, options: ProcessorOptions
-) -> BaseFrame | None:
+    frame: Frame | None, options: ProcessorOptions, recursive: bool = True
+) -> Frame | None:
     """
     Combines consecutive 'self time' frames.
     """
@@ -161,10 +161,10 @@ def merge_consecutive_self_time(
     previous_self_time_frame = None
 
     for child in frame.children:
-        if isinstance(child, SelfTimeFrame):
+        if child.identifier == SELF_TIME_FRAME_IDENTIFIER:
             if previous_self_time_frame:
                 # merge
-                previous_self_time_frame.self_time += child.self_time
+                previous_self_time_frame.time += child.time
                 child.remove_from_parent()
             else:
                 # keep a reference, maybe it'll be added to on the next loop
@@ -172,15 +172,16 @@ def merge_consecutive_self_time(
         else:
             previous_self_time_frame = None
 
-    for child in frame.children:
-        merge_consecutive_self_time(child, options=options)
+    if recursive:
+        for child in frame.children:
+            merge_consecutive_self_time(child, options=options, recursive=True)
 
     return frame
 
 
 def remove_unnecessary_self_time_nodes(
-    frame: BaseFrame | None, options: ProcessorOptions
-) -> BaseFrame | None:
+    frame: Frame | None, options: ProcessorOptions
+) -> Frame | None:
     """
     When a frame has only one child, and that is a self-time frame, remove
     that node and move the time to parent, since it's unnecessary - it
@@ -189,10 +190,8 @@ def remove_unnecessary_self_time_nodes(
     if frame is None:
         return None
 
-    if len(frame.children) == 1 and isinstance(frame.children[0], SelfTimeFrame):
-        child = frame.children[0]
-        frame.self_time += child.self_time
-        child.remove_from_parent()
+    if len(frame.children) == 1 and frame.children[0].identifier == SELF_TIME_FRAME_IDENTIFIER:
+        delete_frame_from_tree(frame.children[0], replace_with="nothing")
 
     for child in frame.children:
         remove_unnecessary_self_time_nodes(child, options=options)
@@ -201,8 +200,8 @@ def remove_unnecessary_self_time_nodes(
 
 
 def remove_irrelevant_nodes(
-    frame: BaseFrame | None, options: ProcessorOptions, total_time: float | None = None
-) -> BaseFrame | None:
+    frame: Frame | None, options: ProcessorOptions, total_time: float | None = None
+) -> Frame | None:
     """
     Remove nodes that represent less than e.g. 1% of the output. Options:
 
@@ -214,18 +213,84 @@ def remove_irrelevant_nodes(
         return None
 
     if total_time is None:
-        total_time = frame.time()
+        total_time = frame.time
+
+        # prevent divide by zero
+        if total_time <= 0:
+            total_time = 1e-44
 
     filter_threshold = options.get("filter_threshold", 0.01)
 
     for child in frame.children:
-        proportion_of_total = child.time() / total_time
+        proportion_of_total = child.time / total_time
 
         if proportion_of_total < filter_threshold:
-            frame.self_time += child.time()
-            child.remove_from_parent()
+            delete_frame_from_tree(child, replace_with="nothing")
 
     for child in frame.children:
         remove_irrelevant_nodes(child, options=options, total_time=total_time)
 
     return frame
+
+
+# pylint: disable=W0613
+def remove_first_pyinstrument_frames_processor(
+    frame: Frame | None, options: ProcessorOptions
+) -> Frame | None:
+    """
+    The first few frames when using the command line are the __main__ of
+    pyinstrument, the eval, and the 'runpy' module. I want to remove that from
+    the output.
+    """
+    if frame is None:
+        return None
+
+    # the initial pyinstrument frame
+    def is_initial_pyinstrument_frame(frame: Frame):
+        return (
+            frame.file_path is not None
+            and re.match(r".*pyinstrument[/\\]__main__.py", frame.file_path)
+            and len(frame.children) > 0
+        )
+
+    def is_exec_frame(frame: Frame):
+        return (
+            frame.proportion_of_parent > 0.8
+            and frame.file_path is not None
+            and "<string>" in frame.file_path
+            and len(frame.children) > 0
+        )
+
+    def is_runpy_frame(frame: Frame):
+        return (
+            frame.proportion_of_parent > 0.8
+            and frame.file_path is not None
+            and (re.match(r".*runpy.py", frame.file_path) or "<frozen runpy>" in frame.file_path)
+            and len(frame.children) > 0
+        )
+
+    result = frame
+
+    if not is_initial_pyinstrument_frame(result):
+        return frame
+
+    result = result.children[0]
+
+    if not is_exec_frame(result):
+        return frame
+
+    result = result.children[0]
+
+    if not is_runpy_frame(result):
+        return frame
+
+    # at this point we know we've matched the first few frames of a command
+    # line invocation. We'll trim some runpy frames and return.
+
+    while is_runpy_frame(result):
+        result = result.children[0]
+
+    # remove this frame from the parent to make it the new root frame
+    result.remove_from_parent()
+
+    return result
